//---------------------------------------------------------------------------
// Author: Cato M. Jonassen
// Use: A MUX to switch between the SW and HW PIXEL DATA to the frame buffer
//---------------------------------------------------------------------------
module  HW_SW_MUX(	//HW inputs
					iHW_R, 
					iHW_G,
					iHW_B,
					iHW_V,
					iHW_CLK,
					//SW inputs
					iSW_R,
					iSW_G,
					iSW_B,
					iSW_V,
					iSW_CLK,
					//Input SEL
					iSEL,
					//MUX outputs
					oR,
					oG,
					oB,
					oV,
					oCLK
);

//Input Ports
input	[11:0]		iHW_R;
input	[11:0]		iHW_G;
input	[11:0]		iHW_B;
input				iHW_V;
input				iHW_CLK;

input	[11:0]		iSW_R;
input	[11:0]		iSW_G;
input	[11:0]		iSW_B;
input				iSW_V;
input				iSW_CLK;

input 				iSEL;

//Output Ports
output 	[11:0]		oR;
output 	[11:0]		oG;
output 	[11:0]		oB;
output				oV;
output				oCLK;

//Output registers
reg		[11:0]		R;
reg		[11:0]		G;
reg		[11:0]		B;
reg					V;
reg					CLK;

//Assignment of registers to output
assign	oR		=	R[11:0];
assign	oG		=	G[11:0];
assign	oB		=	B[11:0];
assign	oV		=	V;
assign	oCLK	=	CLK;

always @ (iSEL or iHW_R or iHW_G or iHW_B or iHW_V or iHW_CLK or iSW_R or iSW_G or iSW_B or iSW_V or iSW_CLK)
begin : MUX
	if (iSEL == 1'b0) 
	begin
      R 	= iHW_R;
      G 	= iHW_G;
      B 	= iHW_B;
      V 	= iHW_V;
      CLK	= iHW_CLK;
	end 
	else 
	begin
      R 	= iSW_R;
      G 	= iSW_G;
      B 	= iSW_B;
      V 	= iSW_V;
      CLK	= iSW_CLK;
	end
end

endmodule //End of module MUX. 