// --------------------------------------------------------------------
// Copyright (c) 2007 by Terasic Technologies Inc. 
// --------------------------------------------------------------------
//
// Permission:
//
//   Terasic grants permission to use and modify this code for use
//   in synthesis for all Terasic Development Boards and Altera Development 
//   Kits made by Terasic.  Other use of this code, including the selling 
//   ,duplication, or modification of any portion is strictly prohibited.
//
// Disclaimer:
//
//   This VHDL/Verilog or C/C++ source code is intended as a design reference
//   which illustrates how these types of functions can be implemented.
//   It is the user's responsibility to verify their design for
//   consistency and functionality through the use of formal
//   verification methods.  Terasic provides no warranty regarding the use 
//   or functionality of this code.
//
// --------------------------------------------------------------------
//           
//                     Terasic Technologies Inc
//                     356 Fu-Shin E. Rd Sec. 1. JhuBei City,
//                     HsinChu County, Taiwan
//                     302
//
//                     web: http://www.terasic.com/
//                     email: support@terasic.com
//
// --------------------------------------------------------------------
//
// Major Functions:	RAW2RGB
//
// --------------------------------------------------------------------
//
// Revision History :
// --------------------------------------------------------------------
//   Ver  :| Author            :| Mod. Date :| Changes Made:
//   V1.0 :| Johnny FAN        :| 07/07/09  :| Initial Revision
// --------------------------------------------------------------------

module RAW2RGB(	oRed,
				oGreen,
				oBlue,
				oDVAL,
				omDATA_0,
				omDATAd_0,
				omDATA_1,
				omDATAd_1,
				oPIX_VALCLK,
				iX_Cont,
				iY_Cont,
				iFUNC,
				iDATA,
				iDVAL,
				iCLK,
				iRST
				);

input	[10:0]	iX_Cont;
input	[10:0]	iY_Cont;
input	[11:0]	iDATA;
input			iDVAL;
input			iCLK;
input			iRST;
input	[2:0]	iFUNC;
output	[11:0]	oRed;
output	[11:0]	oGreen;
output	[11:0]	oBlue;
output			oDVAL;
output	[11:0]	omDATA_0;
output	[11:0]	omDATAd_0;
output	[11:0]	omDATA_1;
output	[11:0]	omDATAd_1;
output			oPIX_VALCLK;
wire	[11:0]	mDATA_0;
wire	[11:0]	mDATA_1;
wire			INV_COLOR;
reg		[11:0]	mDATAd_0;
reg		[11:0]	mDATAd_1;
reg		[11:0]	mCCD_R;
reg		[12:0]	mCCD_G;
reg		[11:0]	mCCD_B;
reg				mDVAL;

//If INV_COLOR == '1' then invert the data else normal color mode
assign	oRed		=	INV_COLOR ? ~mCCD_R[11:0] : mCCD_R[11:0];
//Divides by two because of the addition of the green in the demosaic
assign	oGreen		=	INV_COLOR ? ~mCCD_G[12:1] : mCCD_G[12:1]; 
assign	oBlue		=	INV_COLOR ? ~mCCD_B[11:0] : mCCD_B[11:0];

assign	oDVAL		=	mDVAL;
assign	omDATA_0	= 	mDATA_0[11:0];
assign	omDATAd_0	=	mDATAd_0[11:0];
assign	omDATA_1 	=	mDATA_1[11:0];
assign	omDATAd_1	=	mDATAd_1[11:0];
assign	oPIX_VALCLK	=	iDVAL&iCLK;
assign	INV_COLOR	=	iFUNC[0:0];

//The Line_Buffer is a 2tap shift register with a 1280 clock width between the taps (1280 is the same as the column size from the camera)
//Because of this width the two taps presents two lines simultanious which is used to be able to read 2x2 pixel blocks for the demosiac 
//algorithm.
Line_Buffer 	u0	(	.clken(iDVAL),
						.clock(iCLK),
						.shiftin(iDATA),
						.taps0x(mDATA_1),
						.taps1x(mDATA_0)	);

always@(posedge iCLK or negedge iRST)
begin
	if(!iRST)
	begin
		mCCD_R	<=	0;
		mCCD_G	<=	0;
		mCCD_B	<=	0;
		mDATAd_0<=	0;
		mDATAd_1<=	0;
		mDVAL	<=	0;
	end
	else
	begin
		mDATAd_0	<=	mDATA_0;
		mDATAd_1	<=	mDATA_1;
		//This just gives the frame buffer valid data 1/4 th of the time
		mDVAL		<=	{iY_Cont[0]|iX_Cont[0]}	?	1'b0	:	iDVAL; 
		if({iY_Cont[0],iX_Cont[0]}==2'b10)//Y=odd and X=even => Red data 
		begin
			mCCD_R	<=	mDATA_0;
			mCCD_G	<=	mDATAd_0+mDATA_1;
			mCCD_B	<=	mDATAd_1;
		end	
		else if({iY_Cont[0],iX_Cont[0]}==2'b11)//Y=odd and X=odd => Green1 data
		begin
			mCCD_R	<=	mDATAd_0;
			mCCD_G	<=	mDATA_0+mDATAd_1;
			mCCD_B	<=	mDATA_1;
		end
		//This, when x[0] and y[0] both are equal 00 is the only one that gives data valid out. 
		//The reason for not using all the four combinations is to reduce the output resolution to 640x480 by interpolation 1 of 4 pixels. 
		//This equals a step by two horizontally and a step by two vertically
		else if({iY_Cont[0],iX_Cont[0]}==2'b00)//Y=even and X=even => Green2 data 
		begin
			mCCD_R	<=	mDATA_1;
			mCCD_G	<=	mDATA_0+mDATAd_1;
			mCCD_B	<=	mDATAd_0;
		end
		else if({iY_Cont[0],iX_Cont[0]}==2'b01)//Y=even and X=odd => Blue data
		begin
			mCCD_R	<=	mDATAd_1;
			mCCD_G	<=	mDATAd_0+mDATA_1;
			mCCD_B	<=	mDATA_0;
		end
	end
end

endmodule